@@ -18,10 +18,11 @@
 #
 """PLUSH kind of PyLUceneSHell.
 
-$Id: plush.py 51139 2007-02-19 21:22:12Z bdelbosc $
+$Id: plush.py 51524 2007-04-12 08:25:31Z bdelbosc $
 """
 import sys
 import os
+from stat import ST_MODE, ST_SIZE, S_ISDIR, S_ISREG
 import re
 from sre_compile import error as RegexError
 from datetime import datetime
@@ -65,8 +66,6 @@ def guessEncoding():
         encoding = 'utf-8'
     return encoding
 
-
-
 class StringReader(object):
     """Class for analyzer input."""
     def __init__(self, u_text):
@@ -89,6 +88,41 @@ class StringReader(object):
         """Close"""
         pass
 
+def getTreeSize(top):
+    """Recursively descend the directory tree rooted at top,
+       return total size."""
+    size = 0
+    if not os.path.exists(top):
+        return 0
+    for item in os.listdir(top):
+        pathname = os.path.join(top, item)
+        try:
+            stat = os.stat(pathname)
+        except OSError:
+            continue
+        mode = stat[ST_MODE]
+        if S_ISDIR(mode):
+            # It's a directory, recurse into it
+            size += stat[ST_SIZE]
+            size += getTreeSize(pathname)
+        elif S_ISREG(mode):
+            # It's a file
+            size += stat[ST_SIZE]
+        else:
+            # Unknown file type
+            continue
+    return size
+
+def readableSize(size):
+    """Human readable size."""
+    if size and size < 1024:
+        return '1 K'
+    elif size > 1048576:
+        return '%.02f M' % float(size/1048576.0)
+    elif size:
+        return str(int(size) / 1024) + ' K'
+    return '?'
+
 #------------------------------------------------------------
 # Base class
 #
@@ -121,6 +155,8 @@ class PlushBase:
                       Field.Index.UN_TOKENIZED))
         doc.add(Field("path", '/path/to/dummy.txt', Field.Store.YES,
                       Field.Index.UN_TOKENIZED))
+        doc.add(Field("path", '/path/to/another/dummy.txt', Field.Store.YES,
+                      Field.Index.UN_TOKENIZED))
         doc.add(Field("contents", "foo dummy bar", Field.Store.YES,
                       Field.Index.TOKENIZED))
         writer.addDocument(doc)
@@ -203,15 +239,17 @@ class PlushBase:
         fields = []
         doc = self.getDoc(doc_num)
         for name in self.fields:
-            field = doc.getField(name)
-            if field:
+            mfields= doc.getFields(name)
+            if not mfields:
+                fields.append((name, False, False, False, False, False,
+                               'N/A'))
+                continue
+            for field in mfields:
                 fields.append((field.name(), field.isStored(),
                                field.isIndexed(),
                                field.isTokenized(), field.isBinary(),
                                field.isCompressed(), field.stringValue()))
-            else:
-                fields.append((name, False, False, False, False, False,
-                               'N/A'))
+
         return fields
 
     def search(self, command, field_id="contents", sort_on=None,
@@ -231,7 +269,7 @@ class PlushBase:
         return self.searcher.search(query)
 
     def getTermFreqs(self, field=None, max_term=None, pattern=None):
-        """Return a list of (number of occurence, word) for the field."""
+        """Return a list ([(num occurence, term), ..], number of term)"""
         item = self.index_reader.terms()
         min_freq = 0
         freqs = []
@@ -244,11 +282,13 @@ class PlushBase:
                 pat = re.compile(pattern)
             except RegexError:
                 print "Error: '%s' is an invalid regex" % pattern
-                return []
+                return [], 0
+        count = 0
         while(item.next()):
             term = item.term()
             if field and term.field() != field:
                 continue
+            count += 1
             word = term.text()
             freq = item.docFreq()
             if pattern is not None and not pat.search(word):
@@ -260,9 +300,9 @@ class PlushBase:
             if len(freqs) > limit:
                 freqs.pop()
             min_freq = freqs[0][0]
-
+        item.close()
         freqs = [(-1*freq, word) for freq, word in freqs]
-        return freqs
+        return freqs, count
 
     def initAnalyzers(self):
         """Init all analyzer."""
@@ -396,18 +436,16 @@ Using PyLucene %s and Lucene %s.
         print "Directory info"
         print "--------------"
         print "* Directory path             : %s" % self.store_path
+        store_size = getTreeSize(self.store_path)
+        print "* Directory size             : %s" % readableSize(store_size)
         print "* Directory current version  : %s" % (
             IndexReader.getCurrentVersion(directory))
         print "* Number of docs             : %s (max doc num: %s)" % (
             nb_docs, max_doc)
         print "* Number of fields           : %d" % len(self.fields)
-        print "* Number of terms            :",
-        terms = self.index_reader.terms()
-        nb_terms = 0
-        while terms.next():
-            nb_terms += 1
-        terms.close()
-        print nb_terms
+        if nb_docs:
+            print "* Average doc size           : %s" % (
+                readableSize(store_size / float(nb_docs)))
         try:
             last_modified = datetime.fromtimestamp(
                 IndexReader.lastModified(directory)/1000.0)
@@ -642,7 +680,7 @@ Using PyLucene %s and Lucene %s.
             return
         print "Top %d term occurences for field '%s' matching '%s'." % (
             self.limit, field and field or '*', pattern and pattern or '*')
-        items = self.getTermFreqs(field, self.limit, pattern)
+        items, count = self.getTermFreqs(field, self.limit, pattern)
         if not items:
             print "No match."
             return
@@ -651,6 +689,8 @@ Using PyLucene %s and Lucene %s.
         for freq, term in items:
             print "  %8d %s" % (freq, term)
         print "---------- ----"
+        print "Number of terms for field '%s': %d" % (field and field or '*',
+                                                      count)
 
     def do_history(self, message):
         """\history
